#include "cui.h"
#include "esc.h"
#include "key.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

cui cui_focus = NULL;
int cui_running = FALSE;

cui
cui_alloc(int size)
{
	cui obj;

	if((obj = malloc(size)) == NULL) ERR("No Mem");
	return obj;
}

void
cui_base_init(cui obj, cui parent, int x, int y, int w, int h)
{
	obj->parent = parent;
	obj->children = NULL;
	obj->next = NULL;
	obj->x = x;
	obj->y = y;
	obj->w = w;
	obj->h = h;

	if(parent) cui_base_child_add(parent, obj);
	obj->handler_list = NULL;
	obj->flags = 0;
}

void
cui_base_child_add(cui obj, cui child)
{
	child->next = obj->children;
	obj->children = child;
}

cui
cui_panel_new(cui parent, int x, int y, int w, int h)
{
	cui obj = cui_alloc(sizeof(struct cui_panel));
	cui_panel_init(obj, parent, x, y, w, h);
	return obj;
}

void
cui_panel_init(cui obj, cui parent, int x, int y, int w, int h)
{
	cui_base_init(obj, parent, x, y, w, h);
}

cui
cui_label_new(cui parent, int x, int y, char *s)
{
	cui obj = cui_alloc(sizeof(struct cui_label));
	cui_label_init(obj, parent, x, y, s);
	return obj;
}

void
cui_label_init(cui obj, cui parent, int x, int y, char *s)
{
	cui_label p = (cui_label)obj;
	cui_base_init(obj, parent, x, y, strlen(s), 1);
	p->s = s;	
}

char *
cui_label_str_get(cui obj)
{
	cui_label p = (cui_label)obj;
	return p->s;
}

cui
cui_button_new(cui parent, int x, int y, char *s)
{
	cui obj = cui_alloc(sizeof(struct cui_button));
	cui_button_init(obj, parent, x, y, s);
	return obj;
}

void
cui_button_init(cui obj, cui parent, int x, int y, char *s)
{
	cui_label_init(obj, parent, x, y, s);
	obj->w += 2;
	obj->flags |= CUI_FLG_CAN_FOCUS;
}

char *
cui_button_str_get(cui obj)
{
	return cui_label_str_get(obj);
}

cui_handler_list
cui_handler_list_new(cui_handler hdr, int evt, void *prm, cui_handler_list next)
{
	cui_handler_list list;
	
	if((list = malloc(sizeof(struct cui_handler_list))) == NULL) ERR("No Mem");
	list->hdr = hdr;
	list->evt = evt;
	list->prm = prm;
	list->next = next;
	return list;
}

void
cui_bind(cui obj, int evt, cui_handler hdr, void *prm)
{
	obj->handler_list = cui_handler_list_new(hdr, evt, prm, obj->handler_list);
}

int
cui_handler_call(cui obj, cui_handler_list list, int evt, int val)
{
	if(list == NULL) return FALSE;
	if(cui_handler_call(obj, list->next, evt, val)) return TRUE;
	if(evt != list->evt) return FALSE;
	return (*list->hdr)(obj, evt, val, list->prm);
}

int
cui_gx(cui obj)
{
	return obj == NULL ? 0 : cui_gx(obj->parent) + obj->x;
}

int
cui_gy(cui obj)
{
	return obj == NULL ? 0 : cui_gy(obj->parent) + obj->y;
}

int
cui_glen(cui o1, cui o2)
{
	int dx, dy;

	dx = (cui_gx(o1) + o1->w / 2) - (cui_gx(o2) + o2->w / 2);
	dy = (cui_gy(o1) + o1->h / 2) - (cui_gy(o2) + o2->h / 2);
	return dx * dx + dy * dy;
}

int
cui_focus_move_val(cui obj, int key)
{
	if(cui_focus) return cui_glen(obj, cui_focus);

	switch(key){
	case CUI_KEY_UP:
		return cui_gy(obj);
	case CUI_KEY_DOWN:
		return cui_gy(obj) + obj->h;
	case CUI_KEY_LEFT:
		return cui_gx(obj);
	case CUI_KEY_RIGHT:
		return cui_gx(obj) + obj->w;
	}
	return 0;	
}

cui
cui_focus_move_judge(cui obj, int key, cui old, int *val)
{
	int v, jg = FALSE;

	if(obj == NULL) return old;
	old = cui_focus_move_judge(obj->next, key, old, val);
	old = cui_focus_move_judge(obj->children, key, old, val);

	if(obj == cui_focus) return old;
	if(!(obj->flags & CUI_FLG_CAN_FOCUS)) return old;
	switch(key){
	case CUI_KEY_UP:
	case CUI_KEY_DOWN:
	case CUI_KEY_LEFT:
	case CUI_KEY_RIGHT:
		break;
	default:
		return old;
	}

	v = cui_focus_move_val(obj, key);
	if(!old){
		jg = TRUE;
	}else if(cui_focus){
		switch(key){
		case CUI_KEY_UP:
			jg = cui_gy(obj) <= cui_gy(cui_focus);
			break;
		case CUI_KEY_DOWN:
			jg = cui_gy(obj) >= cui_gy(cui_focus);
			break;
		case CUI_KEY_LEFT:
			jg = cui_gx(obj) <= cui_gx(cui_focus);
			break;
		case CUI_KEY_RIGHT:
			jg = cui_gx(obj) >= cui_gx(cui_focus);
			break;
		}
		jg = jg && v < *val;
	}else{
		switch(key){
		case CUI_KEY_UP:
		case CUI_KEY_LEFT:
			jg = v < *val;
			break;
		case CUI_KEY_DOWN:
		case CUI_KEY_RIGHT:
			jg = v > *val;
			break;
		}
	}
	if(jg) *val = v;
	return jg ? obj : old;
}

void
cui_quit(void)
{
	cui_running = FALSE;
}

void
cui_main(cui top_obj)
{
	int key, val;
	cui obj;

	cui_esc_enter();
	cui_key_enter();

	cui_running = TRUE;	
	while(cui_running){
		if((key = cui_key_get2()) == 0) continue;
		if(cui_focus){
			if(cui_handler_call(cui_focus, cui_focus->handler_list, CUI_EVT_KEY, key)) continue;
		}
		obj = cui_focus_move_judge(top_obj, key, NULL, &val);
		if(obj) cui_focus = obj;
	}
	cui_key_exit();
	cui_esc_exit();
}

/* EOF */
