#ifndef __CUI_H__
#define __CUI_H__

typedef struct cui_base *cui;

#define MSG(s) fprintf(stderr, "%s() %s L%d : %s\n", __func__, __FILE__, __LINE__, s)
#define ERR(s) do{ fprintf(stderr, "ERR "); MSG(s); exit(1); }while(0)

#define FALSE	0
#define TRUE	1

#define CUI_EVT_KEY		(1<<0)
#define CUI_EVT_DRAW		(1<<1)
#define CUI_EVT_BUTTON		(1<<2)

#define CUI_FLG_CAN_FOCUS	(1<<0)

typedef int (*cui_handler)(cui obj, int evt, int val, void *prm);

typedef struct cui_handler_list *cui_handler_list;

struct cui_handler_list{
	cui_handler hdr;
	int evt;
	void *prm;
	cui_handler_list next;
};

struct cui_base{
	cui parent, children, next;
	int x, y, w, h;
	cui_handler_list handler_list;
	int flags;
};	

typedef struct cui_panel{
	struct cui_base base;
} *cui_panel;

typedef struct cui_label{
	struct cui_base base;
	char *s;
} *cui_label;

typedef struct cui_button{
	struct cui_label label;
} *cui_button;

cui cui_alloc(int size);
void cui_base_init(cui obj, cui parent, int x, int y, int w, int h);
void cui_base_child_add(cui obj, cui child);
cui cui_panel_new(cui parent, int x, int y, int w, int h);
void cui_panel_init(cui obj, cui parent, int x, int y, int w, int h);
cui cui_label_new(cui parent, int x, int y, char *s);
void cui_label_init(cui obj, cui parent, int x, int y, char *s);
char *cui_label_str_get(cui obj);
cui cui_button_new(cui parent, int x, int y, char *s);
void cui_button_init(cui obj, cui parent, int x, int y, char *s);
char *cui_button_str_get(cui obj);

void cui_bind(cui obj, int evt, cui_handler hdr, void *prm);
void cui_quit(void);
void cui_main(cui top_obj);

#endif
